require 'yaml'

module DistributedCache
  class Manifest
    attr_reader :config
    attr_accessor :data, :date_format

    DEFAULT_DATE_FORMAT = '%Y%m%d'

    module Field
      CACHE_NAME = 'cache_name'
      DATE = 'date'
      DATE_FORMAT = 'date_format'
      CACHE_VERSION = 'cache_version'
      FILES = 'files'
    end
    include Field

    def initialize(config, name=nil, version=nil)
      @config = config
      @data = {
        CACHE_NAME => name,
        CACHE_VERSION => version,
        FILES => []
      }
      load
    end

    def files
      @data[FILES]
    end

    def cache_name
      @data[CACHE_NAME]
    end

    def cache_version
      @data[CACHE_VERSION]
    end

    def add_file(name, file)
      @data[FILES] << [name, Digest::MD5.file(file).to_s]
    end

    def [](key)
      @data[key]
    end

    def []=(key, value)
      @data[key] = value
    end

    # if the manifest file exists we only need todo a partial update of the cache
    def partial?
      File.exists? manifest_file
    end

    def full?
      ! partial?
    end

    # this will be used as the remote file path and the local bundle directory
    def cache_path
      "#{cache_name}/#{date}/#{cache_version}"
    end

    def date_format
      @date_format || DEFAULT_DATE_FORMAT
    end

    def date
      Time.now.strftime date_format
    end

    def bundle_dir
      "#{config.bundle_dir}/#{cache_path}".tap do |d|
        FileUtils.mkdir_p(d) unless File.exists?(d)
      end
    end

    def cache_dir
      "#{config.cache_dir}/#{cache_name}".tap do |d|
        FileUtils.mkdir_p(d) unless File.exists?(d)
      end
    end

    def manifest_file
      "#{bundle_dir}/manifest.yml"
    end

    def latest_bundle_dir
      "#{config.bundle_dir}/#{cache_name}/latest"
    end

    def save
      File.open(manifest_file, 'w+') do |f|
        f.write @data.to_yaml
      end
    end

    def update_latest_bundle_dir
      File.unlink(latest_bundle_dir) if File.exists?(latest_bundle_dir)
      dir = File.dirname latest_bundle_dir
      Dir.chdir(dir) do
        FileUtils.symlink "#{date}/#{cache_version}", File.basename(latest_bundle_dir)
      end
    end

    def self.load(config, manifest_file)
      new(config).tap do |manifest|
        manifest.data = YAML.load_file(manifest_file)
      end
    end

    private

    def load
      @data = YAML.load_file(manifest_file) if File.exists?(manifest_file)
    end
  end
end
